package javax.sound.sampled;

import java.util.Arrays;

public interface DataLine extends Line
{
    public abstract void drain();
    
    public abstract void flush();
    
    public abstract void start();
    
    public abstract void stop();
    
    public abstract boolean isRunning();
    
    public abstract boolean isActive();
    
    public abstract AudioFormat getFormat();
    
    public abstract int getBufferSize();
    
    public abstract int available();
    
    public abstract int getFramePosition();
    
    public abstract long getLongFramePosition();
    
    public abstract long getMicrosecondPosition();
    
    public abstract float getLevel();
    
    public static class Info extends Line.Info
    {
        private final AudioFormat[] formats;
        private final int minBufferSize;
        private final int maxBufferSize;
        
        public Info(Class<?> clazz, AudioFormat[] original, int minBufferSize, int maxBufferSize) {
            super(clazz);
            if (original == null) {
                this.formats = new AudioFormat[0];
            }
            else {
                this.formats = (AudioFormat[])Arrays.copyOf(original, original.length);
            }
            this.minBufferSize = minBufferSize;
            this.maxBufferSize = maxBufferSize;
        }
        
        public Info(Class<?> clazz, AudioFormat audioFormat, int n) {
            super(clazz);
            if (audioFormat == null) {
                this.formats = new AudioFormat[0];
            }
            else {
                this.formats = new AudioFormat[] { audioFormat };
            }
            this.minBufferSize = n;
            this.maxBufferSize = n;
        }
        
        public Info(Class<?> clazz, AudioFormat audioFormat) {
            this(clazz, audioFormat, -1);
        }
        
        public AudioFormat[] getFormats() {
            return (AudioFormat[])Arrays.copyOf(this.formats, this.formats.length);
        }
        
        public boolean isFormatSupported(AudioFormat audioFormat) {
            int i = 0;
            while (i < this.formats.length) {
                if (audioFormat.matches(this.formats[i])) {
                    return true;
                }
                ++i;
            }
            return false;
        }
        
        public int getMinBufferSize() {
            return this.minBufferSize;
        }
        
        public int getMaxBufferSize() {
            return this.maxBufferSize;
        }
        
        @Override
        public boolean matches(Line.Info info) {
            if (!super.matches(info)) {
                return false;
            }
            DataLine.Info info2 = (DataLine.Info)info;
            if (this.getMaxBufferSize() >= 0 && info2.getMaxBufferSize() >= 0 && this.getMaxBufferSize() > info2.getMaxBufferSize()) {
                return false;
            }
            if (this.getMinBufferSize() >= 0 && info2.getMinBufferSize() >= 0 && this.getMinBufferSize() < info2.getMinBufferSize()) {
                return false;
            }
            AudioFormat[] formats = this.getFormats();
            if (formats != null) {
                int i = 0;
                while (i < formats.length) {
                    if (formats[i] != null && !info2.isFormatSupported(formats[i])) {
                        return false;
                    }
                    ++i;
                }
            }
            return true;
        }
        
        @Override
        public String toString() {
            StringBuffer obj = new StringBuffer();
            if (this.formats.length == 1 && this.formats[0] != null) {
                obj.append(" supporting format " + this.formats[0]);
            }
            else if (this.getFormats().length > 1) {
                obj.append(" supporting " + this.getFormats().length + " audio formats");
            }
            if (this.minBufferSize != -1 && this.maxBufferSize != -1) {
                obj.append(", and buffers of " + this.minBufferSize + " to " + this.maxBufferSize + " bytes");
            }
            else if (this.minBufferSize != -1 && this.minBufferSize > 0) {
                obj.append(", and buffers of at least " + this.minBufferSize + " bytes");
            }
            else if (this.maxBufferSize != -1) {
                obj.append(", and buffers of up to " + this.minBufferSize + " bytes");
            }
            return new String(super.toString() + obj);
        }
    }
}
